#!/usr/bin/perl -w

# $Id: SNMPHostQueryData.pl,v 1.4 2005/06/09 01:37:31 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Net::SNMP; # Tested on v3.6, on Debian Woody.
use Getopt::Long qw( :config no_ignore_case );
use Parallel::ForkManager;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 %Oids
			 $ForkManager
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 Community       => 'public', # Default community
			 Port            => '161',    # Default port
			 MaxForks        => 1,   # Effectively disable forking
			 Append          => 0,   # Overwrite hash values or append
			 );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugMsg('Beginning SNMP scanning...');

# Setup forking
DebugMsg("Using Parallel ForkManager, maximum $Options{'MaxForks'} children.");

$ForkManager = new Parallel::ForkManager( $Options{'MaxForks'} );

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp();
	
	# engage a new fork.
	my $pid = $ForkManager->start and next;
	
	# split out hostname
	my $InputHostName = ( split(/:/, $_) )[0];

	# hash out any additional host data
	my $InputHostData = CreateHashFromString( '[A-Z][A-Z]', $_ );
	
	SNMPQueryHost( $InputHostName, $InputHostData );
	
	# End fork
	$ForkManager->finish;
	
};

# Wait til all forks complete
$ForkManager->wait_all_children;

######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'c=s'       => \$Options{'Community'},
				'p=i'       => \$Options{'Port'},
				'f=i'       => \$Options{'MaxForks'},
				'a'         => \$Options{'Append'},
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# SNMPQueryHost

sub SNMPQueryHost {

	my ( $HostName, $HostData ) = @_ ;

	DebugMsg("Scanning $HostName");

	my ( $SNMPSession , $SNMPError, $SNMPResponse,
		 $RemoteData, $OutputString );

	# Create session object (does NOT open a connection, its stateless)
	# What about version, timeout, retries, debug mode?
	( $SNMPSession , $SNMPError ) = Net::SNMP->session( 
									   -hostname    => $HostName,
									   -community   => $Options{'Community'},
									   -port        => $Options{'Port'},
									   );

	# Check session object was created
	if ( ! defined $SNMPSession ) {
		print STDERR "$SNMPError\n";
		return undef;
	};
		
	########################################
	# Query given remote tags
	$RemoteData = GetConfigurationDirectives( $SNMPSession , '..' );

	# Finished with SNMP.
	$SNMPSession->close();

	# Done pulling configuration directives from remote host
	########################################################

	DebugDumpHash( "RemoteData for $HostName" , $RemoteData );

	# Overwrite, unless told to append
	my $OutputHash;

	if ( ! $Options{'Append'} ) {
		
		$OutputHash = MergeHash( $HostData, $RemoteData );

	} else {

		$OutputHash = AddHash( $HostData, $RemoteData );

	};

	my $OutputRecord = CreateStringFromHash( $OutputHash );

	# Print record

	print $HostName . $OutputRecord . "\n";

};

# End SNMPScanHost
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

SNMPHostQueryData.pl

Reads hostnames or IP addresses via STDIN. Performs an SNMP get on
each host for sysContact and sysLocation, and parses it for NACE
configuration directives. If directives exist, the host address and
any directives are printed to STDOUT as a colon delimited list.

Sample Output:

192.168.1.1:HG=HomeGroup:DS=Web Server:PN=router1

Syntax:

	-c <community>

		SNMP community to use when polling, defaults to public.

	-p <port>

		SNMP port to use when polling, defaults to 169.

	-a

        By default, we overwrite any data in the host record read on
		the command line. This flag triggers append instead.

	-f <forks>

		Maximum number of forks to spawn to scan hosts, defaults to 1.

	-D

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
