#!/usr/bin/perl -w

# $Id: WriteHosts.pl,v 1.2 2005/06/05 06:19:23 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Getopt::Long qw( :config no_ignore_case );
use File::Path;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 %Defaults
			 %Additions
			 %HostKeyMapping
			 %AdditionalKeyMapping
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 OutputDir       => '.', # Create files in current directory 
			 );

########################################
# Defaults Host Hash Values

# Basic hash with values that can be optionally overridden via
# configuration options on the host.

%Defaults = ( HT => undef,        # HostTemplate to "use", required
			  HG => undef,        # HostGroup is required
			  DS => '@@@HOST@@@', # Description for the host w/ subst,
								  #   defaults to hostname
			  PN => undef,        # No default ParentNode
			  );

########################################
# Additional Options

%Additions = ( HG  => undef, # Additional HostGroups
			   PN  => undef, # Additional parents
			   );

########################################
# Output key mapping
# maps each host key to a nagios keyword.
%HostKeyMapping = ( HT => 'use',
					HN => 'host_name',
					AD => 'address',
					HG => '#	host_group', # Not a real nagiosfield
					DS => 'alias',
					PN => 'parents',
					);

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugDumpHash( "Default Host Key Mappings" , \%HostKeyMapping );

DebugDumpHash( "Additional Host Key Mappings" , \%AdditionalKeyMapping );

my $HostKeyMappingPtr = MergeHash( \%HostKeyMapping , \%AdditionalKeyMapping );

DebugDumpHash( "Merged Host Key Mappings" , $HostKeyMappingPtr );

DebugMsg('Creating host records');

my $HostKeys = join( '|' , sort keys %$HostKeyMappingPtr );

DebugMsg("Valid keys are: $HostKeys");

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp;

	# Read input record

	my $RemoteData = CreateHashFromString($HostKeys,$_);

	my $HostName = ( split(/:/) )[0];

	DebugDumpHash( "Defaults for $HostName" , \%Defaults );

	DebugDumpHash( "RemoteData for $HostName" , $RemoteData );

	# Add Hostname into our hash, inheriting Defaults
	
	my $HostData = MergeHash( \%Defaults , 
							  { 
								  'HN' => $HostName , 
								  'AD' => $HostName 
								  }
							  );

	# Override Defaults with RemoteData collected

	$HostData = MergeHash( $HostData , $RemoteData );

	DebugDumpHash( "HostData for $HostName after merge" , $HostData );

	# Add the Additions
	$HostData = AddHash( $HostData , \%Additions  );
	
	DebugDumpHash( "HostData for $HostName after add" , $HostData );

	# Perform substitution on the result.
	$HostData = PerformSubst( $HostData , '\@\@\@HOST\@\@\@' , $HostName );
	
	DebugDumpHash( "HostData for $HostName after subst" , $HostData );

	DebugDumpHash( "HostData for $HostName after includes" , $HostData );
	
	# Write our host record
	WriteRecord( $HostData , 'host' , $HostKeyMappingPtr , 
				 "> $Options{'OutputDir'}/$HostName.cfg" );

};


######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'O=s'       => \$Options{'OutputDir'},
				'k=s'       => \%AdditionalKeyMapping,
				'd=s'       => \%Defaults,
				'a=s'       => \%Additions,
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

	###########################################
	# Check for output dir and create if needed

	-d $Options{'OutputDir'} || mkpath( $Options{'OutputDir'} );


    ########################################
    # Check for required options

	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

WriteHosts.pl

Reads hostnames or IP addresses and optional configuration fields via
STDIN. Writes Nagios configuration files for each host to output
directory.

Syntax:

	-O <Directory>

		Output directory, defaults to current directory.

    -k <key=value>

        Insert key and value into the host key mapping hash.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
