#!/usr/bin/perl -w

# $Id: WriteServices.pl,v 1.2 2005/06/05 06:19:50 root Exp $	

######################################################################
# NACE v0.4
#
# Copyright 2005 Russell Adams <rladams@adamsinfoserv.com>,
# and Adams Information Services.
#
# Licensed under GPL v2. See the README.
#
######################################################################


######################################################################
# Includes

# Use our configuration library, if not in the current directory,
# it's path had best be in the PERLLIB environment variable.
use lib "../lib";
use NACELib; 

# Other requirements
use strict;
use Getopt::Long qw( :config no_ignore_case );
use Digest::MD5 qw( md5_hex );
use File::Path;

# End Includes
######################################################################


######################################################################
# Global vars

use vars qw( 
			 %Options
			 %Defaults
			 %Additions
			 @Targets
			 %ServiceKeyMapping
			 %AdditionalKeyMapping
			 );

# End Global vars
######################################################################


######################################################################
# Default Options

%Options = ( DisplayHelp     => 0,   # Do we show help and exit?
			 OutputDir       => '.', # Create files in current directory 
			 );

########################################
# Defaults service Hash Values

# Basic hash with values that can be optionally overridden via
# configuration options on the host.

%Defaults = ( ST => undef,          # ServiceTemplate to "use", required
			  CG => undef,          # ContactGroup is required
			  SD => undef,          # Description for the service
			  SC => undef,          # Service check command, required
			  );

########################################
# Additional Options

%Additions = ( CG  => undef, # Additional ContactGroups
			   );

########################################
# Output key mapping
# maps each host key to a nagios keyword.
%ServiceKeyMapping = ( ST => 'use',
					   SD => 'service_description',
					   SC => 'check_command',
					   CG => 'contact_groups',
					   HN => 'host_name',
					   );

# End of default options
######################################################################


######################################################################
# Main Program
######################################################################

ProcessOptions();

DebugDumpHash( "Default Service Key Mappings" , \%ServiceKeyMapping );

DebugDumpHash( "Additional Service Key Mappings" , \%AdditionalKeyMapping );

my $ServiceKeyMappingPtr = MergeHash( \%ServiceKeyMapping , \%AdditionalKeyMapping );

DebugDumpHash( "Merged Service Key Mappings" , $ServiceKeyMappingPtr );

DebugMsg('Creating service records');

my $ServiceKeys = join( '|' , sort keys %$ServiceKeyMappingPtr );

DebugMsg("Valid keys are: $ServiceKeys");

# read host list from STDIN
DebugMsg('Reading hosts from STDIN.');

while ( <STDIN> ) {

	chomp;

	# Read input record

	my $RemoteData = CreateHashFromString($ServiceKeys,$_);

	my $HostName = ( split(/:/) )[0];

	DebugDumpHash( "Defaults for $HostName" , \%Defaults );

	DebugDumpHash( "RemoteData for $HostName" , $RemoteData );

	# Add Hostname into our hash, inheriting Defaults
	
	my $ServiceData = MergeHash( \%Defaults , 
							  { 
								  'HN' => $HostName , 
							  }
							  );

	# Override Defaults with RemoteData collected

	$ServiceData = MergeHash( $ServiceData , $RemoteData );

	DebugDumpHash( "ServiceData for $HostName after merge" , $ServiceData );

	# Add the Additions
	$ServiceData = AddHash( $ServiceData , \%Additions  );
	
	DebugDumpHash( "ServiceData for $HostName after add" , $ServiceData );

	# Perform substitution on the result.
	$ServiceData = PerformSubst( $ServiceData , '\@\@\@HOST\@\@\@' , $HostName );
	
	DebugDumpHash( "ServiceData for $HostName after subst" , $ServiceData );

	DebugDumpHash( "ServiceData for $HostName after includes" , $ServiceData );
	
	# Write our host record
	WriteRecord( $ServiceData , 'service' , $ServiceKeyMappingPtr , 
				 "> $Options{'OutputDir'}/$HostName" . "_" . md5_hex( $ServiceData->{'SC'} ) . ".cfg" );

};


######################################################################
# End Main Program
######################################################################


######################################################################
# Functions
######################################################################


######################################################################
# ProcessOptions

sub ProcessOptions {

	GetOptions( 
				'D'         => \$Debug,
				'h'         => \$Options{'DisplayHelp'},
				'O=s'       => \$Options{'OutputDir'},
				'k=s'       => \%AdditionalKeyMapping,
				'd=s'       => \%Defaults,
				'a=s'       => \%Additions,
				);

	$Debug && DebugDumpHash( 'Options', \%Options );

	$Options{'DisplayHelp'} && DisplayHelp();

	###########################################
	# Check for output dir and create if needed

	-d $Options{'OutputDir'} || mkpath( $Options{'OutputDir'} );

    ########################################
    # Check for required options

	DebugMsg('Options processing complete.');

};

# End ProcessOptions
######################################################################


######################################################################
# DisplayHelp

sub DisplayHelp {

	print <<EOF;

WriteServices.pl

Reads hostnames or IP addresses and optional configuration fields via
STDIN. Writes Nagios configuration files for each service to output
directory.

Syntax:

	-O <Directory>

		Output directory, defaults to current directory.

    -k <key=value>

        Insert key and value into the service key mapping hash.

	-D 

		Debug, enables very verbose debugging information.

EOF

	exit -1;

};

# End DisplayHelp
######################################################################
